@@ -23,6 +23,7 @@
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
+#include "bashline.h"
 
 #if defined(BUFSIZ) && BUFSIZ >= 131072
 #define SSH_BUFSIZ BUFSIZ
@@ -687,8 +688,10 @@ static int do_read(const char* infile, FILE* fp, getmode_t mode,
 {
   if (gvSSHTrySCP && !offset)
   {
+    char* escaped = bash_backslash_quote(infile);
     /* try to set up a scp connection */
-    ssh_scp scp = ssh_scp_new(ftp->session, SSH_SCP_READ, infile);
+    ssh_scp scp = ssh_scp_new(ftp->session, SSH_SCP_READ, escaped);
+    free(escaped);
     if (scp != NULL)
     {
       int rc = ssh_scp_init(scp);
