@@ -66,35 +66,32 @@ extern int get_sort_func(const void *a, const void *b);
 
 static void print_fxp_syntax(void)
 {
-	printf(_("Transfers files between two remote servers.  Usage:\n"
-			 "  fxp [options] files\n"
-			 "Options:\n"
-			 "  -a, --append         append if destination exists\n"
-			 "  -D, --delete-after   delete source file after successful transfer\n"
-			 "      --dir-mask=GLOB  enter only directories matching GLOB pattern\n"
-			 "      --dir-rx-mask=REGEXP\n"
-			 "                       enter only directories matching REGEXP pattern\n"
-			 "  -f, --force          overwrite existing destinations, never prompt\n"
-			 "  -e, --skip-empty     skip empty files\n"
-			 "  -H, --nohup          transfer files in background (nohup mode), quits yafc\n"
-			 "  -i, --interactive    prompt before each transfer\n"
-			 "  -L, --logfile=FILE   use FILE as logfile instead of ~/.yafc/nohup/nohup.<pid>\n"
-			 "  -m, --mask=GLOB      only transfer files matching GLOB pattern\n"
-			 "  -M, --rx-mask=REGEXP only transfer files matching REGEXP pattern\n"
-			 "  -n, --newer          transfer file if destination is newer than source file\n"
-			 "  -o, --output=DEST    store in destination file/directory DEST\n"
-			 "  -p, --preserve       try to preserve file attributes\n"
-			 "  -P, --parents        append source path to destination\n"
-			 "  -q, --quiet          overrides --verbose\n"
-			 "  -r, --recursive      transfer directories recursively\n"
-			 "  -R, --resume         resume broken download (restart at eof)\n"
-			 "  -s, --skip-existing  skip file if destination exists\n"
-			 "  -t, --tagged         transfer tagged files\n"
-			 "  -T, --target=HOST    specify target host\n"
-			 "      --type=TYPE      specify transfer type, 'ascii' or 'binary'\n"
-			 "  -u, --unique         store in unique filename (if target supports STOU)\n"
-			 "  -v, --verbose        explain what is being done\n"
-			 "      --help           display this help\n"));
+	show_help(_("Transfers files between two remote servers."), "fxp [options] files",
+	  _("  -a, --append         append if destination exists\n"
+			"  -D, --delete-after   delete source file after successful transfer\n"
+			"      --dir-mask=GLOB  enter only directories matching GLOB pattern\n"
+			"      --dir-rx-mask=REGEXP\n"
+			"                       enter only directories matching REGEXP pattern\n"
+			"  -f, --force          overwrite existing destinations, never prompt\n"
+			"  -e, --skip-empty     skip empty files\n"
+			"  -H, --nohup          transfer files in background (nohup mode), quits yafc\n"
+			"  -i, --interactive    prompt before each transfer\n"
+			"  -L, --logfile=FILE   use FILE as logfile instead of ~/.yafc/nohup/nohup.<pid>\n"
+			"  -m, --mask=GLOB      only transfer files matching GLOB pattern\n"
+			"  -M, --rx-mask=REGEXP only transfer files matching REGEXP pattern\n"
+			"  -n, --newer          transfer file if destination is newer than source file\n"
+			"  -o, --output=DEST    store in destination file/directory DEST\n"
+			"  -p, --preserve       try to preserve file attributes\n"
+			"  -P, --parents        append source path to destination\n"
+			"  -q, --quiet          overrides --verbose\n"
+			"  -r, --recursive      transfer directories recursively\n"
+			"  -R, --resume         resume broken download (restart at eof)\n"
+			"  -s, --skip-existing  skip file if destination exists\n"
+			"  -t, --tagged         transfer tagged files\n"
+			"  -T, --target=HOST    specify target host\n"
+			"      --type=TYPE      specify transfer type, 'ascii' or 'binary'\n"
+			"  -u, --unique         store in unique filename (if target supports STOU)\n"
+			"  -v, --verbose        explain what is being done\n"));
 }
 
 static bool fxp_exclude_func(rfile *f)
@@ -181,12 +178,21 @@ static int fxpfile(const rfile *fi, unsigned int opt,
 
 	if(test(opt, FXP_PARENTS)) {
 		char *p = base_dir_xptr(fi->path);
-		asprintf(&dest, "%s/%s/%s", output, p, base_name_ptr(fi->path));
+		if (asprintf(&dest, "%s/%s/%s", output, p, base_name_ptr(fi->path)) == -1)
+    {
+      fprintf(stderr, _("Failed to allocate memory.\n"));
+      free(p);
+      return -1;
+    }
 		free(p);
 	} else if(test(opt, FXP_OUTPUT_FILE))
 		dest = xstrdup(output);
 	else
-		asprintf(&dest, "%s/%s", output, base_name_ptr(fi->path));
+		if (asprintf(&dest, "%s/%s", output, base_name_ptr(fi->path)) == -1)
+    {
+      fprintf(stderr, _("Failed to allocate memory.\n"));
+      return -1;
+    }
 
 	path_collapse(dest);
 
@@ -200,7 +206,7 @@ static int fxpfile(const rfile *fi, unsigned int opt,
 	free(q_dest_dir);
 	free(dest_dir);
 	if(r == -1) {
-		transfer_mail_msg(_("failed to create directory %s\n"), dest_dir);
+		transfer_mail_msg(_("Couldn't create directory: %s\n"), dest_dir);
 		free(dpath);
 		free(dest);
 		ftp_use(thisftp);
@@ -426,10 +432,6 @@ static void fxpfiles(list *gl, unsigned int opt, const char *output)
 
 		if(risdir(fp)) {
 			if(test(opt, FXP_RECURSIVE)) {
-				char *recurs_output;
-				char *recurs_mask, *q_recurs_mask;
-				list *rgl;
-
 				if((fxp_dir_glob_mask
 					&& fnmatch(fxp_dir_glob_mask,
 							   base_name_ptr(fp->path),
@@ -443,15 +445,32 @@ static void fxpfiles(list *gl, unsigned int opt, const char *output)
 					{
 						/*printf("skipping %s\n", fp->path);*/
 					} else {
+            char* recurs_output = NULL;
+            bool success = true;
 						if(!test(opt, FXP_PARENTS))
-							asprintf(&recurs_output, "%s/%s",
-									 output ? output : ".", ofile);
+							success = asprintf(&recurs_output, "%s/%s",
+									 output ? output : ".", ofile) != -1;
 						else
-							asprintf(&recurs_output, "%s",
-									 output ? output : ".");
-						rgl = rglob_create();
-						asprintf(&recurs_mask, "%s/*", opath);
-						q_recurs_mask = bash_backslash_quote(recurs_mask);
+							success = asprintf(&recurs_output, "%s",
+									 output ? output : ".") != -1;
+            if (!success)
+            {
+              fprintf(stderr, _("Failed to allocate memory.\n"));
+              transfer_nextfile(gl, &li, true);
+			        continue;
+            }
+
+            char* recurs_mask = NULL;
+            if (asprintf(&recurs_mask, "%s/*", opath) == -1)
+            {
+              free(recurs_output);
+              fprintf(stderr, _("Failed to allocate memory.\n"));
+              transfer_nextfile(gl, &li, true);
+			        continue;
+            }
+
+						char* q_recurs_mask = bash_backslash_quote(recurs_mask);
+            list* rgl = rglob_create();
 						rglob_glob(rgl, q_recurs_mask, true, true,
 								   fxp_exclude_func);
 						free(q_recurs_mask);
