@@ -46,38 +46,35 @@ static bool get_dir_rx_mask_set = false;
 
 static void print_get_syntax(void)
 {
-    printf(_("Receives files from remote.  Usage:\n"
-             "  get [options] files\n"
-             "Options:\n"
-             "  -a, --append         append if destination exists\n"
-             "  -c, --chmod=PERM     change mode of transferred files to PERM\n"
-             "      --chgrp=GROUP    change group of transferred files to GROUP\n"
-             "  -d, --no-dereference copy symbolic links as symbolic links\n"
-             "  -D, --delete-after   delete remote file after successful transfer\n"
-             "      --dir-mask=GLOB  enter only directories matching GLOB pattern\n"
-             "      --dir-rx-mask=REGEXP\n"
-             "                       enter only directories matching REGEXP pattern\n"
-             "  -f, --force          overwrite existing destinations, never prompt\n"
-             "  -e, --skip-empty     skip empty files\n"
-             "  -H, --nohup          transfer files in background (nohup mode), quits yafc\n"
-             "  -i, --interactive    prompt before each transfer\n"
-             "  -L, --logfile=FILE   use FILE as logfile instead of ~/.yafc/nohup/nohup.<pid>\n"
-             "  -m, --mask=GLOB      get only files matching GLOB pattern\n"
-             "  -M, --rx-mask=REGEXP get only files matching REGEXP pattern\n"
-             "  -n, --newer          get file if remote is newer than local file\n"
-             "  -o, --output=DEST    store in local file/directory DEST\n"
-             "  -p, --preserve       try to preserve file attributes and timestamps\n"
-             "  -P, --parents        append source path to destination\n"
-             "  -q, --quiet          overrides --verbose\n"
-             "  -r, --recursive      get directories recursively\n"
-             "  -R, --resume         resume broken download (restart at eof)\n"
-             "  -s, --skip-existing  skip file if destination exists\n"
-             "  -S, --stats[=NUM]    set stats transfer threshold; default is always\n"
-             "  -t, --tagged         transfer tagged file(s)\n"
-             "      --type=TYPE      specify transfer type, 'ascii' or 'binary'\n"
-             "  -u, --unique         always store as unique local file\n"
-             "  -v, --verbose        explain what is being done\n"
-             "      --help           display this help\n"));
+  show_help(_("Receives files from remote."), "get [options] files",
+    _("  -a, --append         append if destination exists\n"
+      "  -c, --chmod=PERM     change mode of transferred files to PERM\n"
+      "      --chgrp=GROUP    change group of transferred files to GROUP\n"
+      "  -d, --no-dereference copy symbolic links as symbolic links\n"
+      "  -D, --delete-after   delete remote file after successful transfer\n"
+      "      --dir-mask=GLOB  enter only directories matching GLOB pattern\n"
+      "      --dir-rx-mask=REGEXP\n"
+      "                       enter only directories matching REGEXP pattern\n"
+      "  -f, --force          overwrite existing destinations, never prompt\n"
+      "  -e, --skip-empty     skip empty files\n"
+      "  -H, --nohup          transfer files in background (nohup mode), quits yafc\n"
+      "  -i, --interactive    prompt before each transfer\n"
+      "  -L, --logfile=FILE   use FILE as logfile instead of ~/.yafc/nohup/nohup.<pid>\n"
+      "  -m, --mask=GLOB      get only files matching GLOB pattern\n"
+      "  -M, --rx-mask=REGEXP get only files matching REGEXP pattern\n"
+      "  -n, --newer          get file if remote is newer than local file\n"
+      "  -o, --output=DEST    store in local file/directory DEST\n"
+      "  -p, --preserve       try to preserve file attributes and timestamps\n"
+      "  -P, --parents        append source path to destination\n"
+      "  -q, --quiet          overrides --verbose\n"
+      "  -r, --recursive      get directories recursively\n"
+      "  -R, --resume         resume broken download (restart at eof)\n"
+      "  -s, --skip-existing  skip file if destination exists\n"
+      "  -S, --stats[=NUM]    set stats transfer threshold; default is always\n"
+      "  -t, --tagged         transfer tagged file(s)\n"
+      "      --type=TYPE      specify transfer type, 'ascii' or 'binary'\n"
+      "  -u, --unique         always store as unique local file\n"
+      "  -v, --verbose        explain what is being done\n"));
 }
 
 static bool get_exclude_func(rfile *f)
@@ -196,7 +193,12 @@ static int getfile(const rfile *fi, unsigned int opt,
 
     if(test(opt, GET_PARENTS)) {
         char *apath = base_dir_xptr(fi->path);
-        asprintf(&dest, "%s%s/%s", output, apath, destname);
+        if (asprintf(&dest, "%s%s/%s", output, apath, destname) == -1)
+        {
+          free(apath);
+          fprintf(stderr, _("Failed to allocate memory.\n"));
+          return -1;
+        }
         free(apath);
     } else {
         /* check if -o option is given, if GET_OUTPUT_FILE is set, we only
@@ -215,7 +217,11 @@ static int getfile(const rfile *fi, unsigned int opt,
         if(dest_is_file)
             dest = xstrdup(output);
         else
-            asprintf(&dest, "%s/%s", output, destname);
+            if (asprintf(&dest, "%s/%s", output, destname) == -1)
+            {
+              fprintf(stderr, _("Failed to allocate memory.\n"));
+              return -1;
+            }
     }
 
     /* make sure destination directory exists */
@@ -226,7 +232,7 @@ static int getfile(const rfile *fi, unsigned int opt,
         if(destdir) {
             r = make_path(destdir, S_IRWXU, S_IRWXU, -1, -1, 1, 0);
             if(r != 0) {
-                transfer_mail_msg(_("failed to create directory %s\n"),
+                transfer_mail_msg(_("Couldn't create directory: %s\n"),
                                   destdir);
                 free(destdir);
                 return -1;
@@ -524,14 +530,27 @@ static void getfiles(list *gl, unsigned int opt, const char *output)
                     } else {
                         char *q_recurs_mask;
 
+                        bool success = true;
                         if(!test(opt, GET_PARENTS))
-                            asprintf(&recurs_output, "%s/%s",
-                                     output ? output : ".", ofile);
+                            success = asprintf(&recurs_output, "%s/%s",
+                                     output ? output : ".", ofile) != -1;
                         else
-                            asprintf(&recurs_output, "%s",
-                                     output ? output : ".");
+                            success = asprintf(&recurs_output, "%s",
+                                     output ? output : ".") != -1;
+                        if (!success)
+                        {
+                          fprintf(stderr, _("Failed to allocate memory.\n"));
+                          transfer_nextfile(gl, &li, true);
+			                    continue;
+                        }
+                        if (asprintf(&recurs_mask, "%s/*", opath) == -1)
+                        {
+                          free(recurs_output);
+                          fprintf(stderr, _("Failed to allocate memory.\n"));
+                          transfer_nextfile(gl, &li, true);
+                          continue;
+                        }
                         rgl = rglob_create();
-                        asprintf(&recurs_mask, "%s/*", opath);
                         q_recurs_mask = bash_backslash_quote(recurs_mask);
                         rglob_glob(rgl, q_recurs_mask, true, true, get_exclude_func);
                         free(q_recurs_mask);
